@@ -74,6 +74,8 @@ class MetatraderAccountDto(TypedDict):
     """MetaTrader account number."""
     server: str
     """MetaTrader server which hosts the account."""
+    version: int
+    """MT version (allowed values are 4 and 5)."""
     provisioningProfileId: str
     """Id of the account's provisioning profile."""
     application: str
@@ -99,6 +101,12 @@ class MetatraderAccountDto(TypedDict):
     """Extra information which can be stored together with your account."""
     reliability: str
     """Used to increase the reliability of the account. Allowed values are regular and high. Default is regular."""
+    baseCurrency: str
+    """3-character ISO currency code of the account base currency. Default value is USD.
+    The setting is to be used for copy trading accounts which use national currencies only, such as some Brazilian
+    brokers. You should not alter this setting unless you understand what you are doing."""
+    copyFactoryRoles: List[str]
+    """Account roles for CopyFactory2 application. Allowed values are `PROVIDER` and `SUBSCRIBER`."""
 
 
 class NewMetatraderAccountDto(TypedDict):
@@ -136,6 +144,10 @@ class NewMetatraderAccountDto(TypedDict):
     """Extra information which can be stored together with your account."""
     reliability: str
     """Used to increase the reliability of the account. Allowed values are regular and high. Default is regular."""
+    baseCurrency: str
+    """3-character ISO currency code of the account base currency. Default value is USD.
+    The setting is to be used for copy trading accounts which use national currencies only, such as some Brazilian
+    brokers. You should not alter this setting unless you understand what you are doing."""
 
 
 class MetatraderAccountUpdateDto(TypedDict):
@@ -358,3 +370,25 @@ class MetatraderAccountClient(MetaApiClient):
             'body': account
         }
         return await self._httpClient.request(opts)
+
+    async def increase_reliability(self, id: str):
+        """Increases MetaTrader account reliability. The account will be temporary stopped to perform this action. (see
+        https://metaapi.cloud/docs/provisioning/api/account/increaseReliability/).
+        Method is accessible only with API access token.
+
+        Args:
+            id: MetaTrader account id.
+
+        Returns:
+            A coroutine resolving when MetaTrader account reliability is increased.
+        """
+        if self._is_not_jwt_token():
+            return self._handle_no_access_exception('increase_reliability')
+        opts = {
+            'url': f'{self._host}/users/current/accounts/{id}/increase-reliability',
+            'method': 'POST',
+            'headers': {
+                'auth-token': self._token
+            }
+        }
+        return await self._httpClient.request(opts)
