@@ -8,12 +8,14 @@ import asyncio
 import functools
 import shutil
 from copy import deepcopy
-from ...metaApi.models import date
+from ...metaApi.models import date, format_error
 
 
 class PacketLoggerOpts(TypedDict):
     """Packet logger options."""
 
+    enabled: Optional[bool]
+    """Whether packet logger is enabled."""
     fileNumberLimit: Optional[int]
     """Maximum amount of files per account. Default is 12."""
     logFileSizeInHours: Optional[float]
@@ -39,7 +41,7 @@ class PacketLogger:
         self._compressSpecifications = opts['compressSpecifications'] if 'compressSpecifications' in opts else True
         self._compressPrices = opts['compressPrices'] if 'compressPrices' in opts else True
         self._previousPrices = {}
-        self._lastKeepAlive = {}
+        self._lastSNPacket = {}
         self._writeQueue = {}
         self._root = './.metaapi/logs'
         self._recordInterval: asyncio.Task or None = None
@@ -66,10 +68,10 @@ class PacketLogger:
             self._writeQueue[packet['accountId']] = {'isWriting': False, 'queue': []}
         if packet['type'] == 'status':
             return
-        if packet['accountId'] not in self._lastKeepAlive:
-            self._lastKeepAlive[packet['accountId']] = {}
-        if packet['type'] == 'keepalive':
-            self._lastKeepAlive[packet['accountId']][instance_index] = packet
+        if packet['accountId'] not in self._lastSNPacket:
+            self._lastSNPacket[packet['accountId']] = {}
+        if packet['type'] in ['keepalive', 'noop']:
+            self._lastSNPacket[packet['accountId']][instance_index] = packet
             return
         queue: List = self._writeQueue[packet['accountId']]['queue']
         if packet['accountId'] not in self._previousPrices:
@@ -93,10 +95,10 @@ class PacketLogger:
                 if prev_price is not None:
                     valid_sequence_numbers = [prev_price['last']['sequenceNumber'],
                                               prev_price['last']['sequenceNumber'] + 1]
-                    if instance_index in self._lastKeepAlive[packet['accountId']] and 'sequenceNumber' in \
-                            self._lastKeepAlive[packet['accountId']][instance_index]:
+                    if instance_index in self._lastSNPacket[packet['accountId']] and 'sequenceNumber' in \
+                            self._lastSNPacket[packet['accountId']][instance_index]:
                         valid_sequence_numbers.append(
-                            self._lastKeepAlive[packet['accountId']][instance_index]['sequenceNumber'] + 1)
+                            self._lastSNPacket[packet['accountId']][instance_index]['sequenceNumber'] + 1)
                     if packet['sequenceNumber'] not in valid_sequence_numbers:
                         self._record_prices(packet['accountId'], instance_index)
                         self._ensure_previous_price_object(packet['accountId'])
@@ -174,22 +176,22 @@ class PacketLogger:
         self._deleteOldLogsInterval.cancel()
         self._deleteOldLogsInterval = None
 
-    def _record_prices(self, account_id: str, instance_index: int):
+    def _record_prices(self, account_id: str, instance_number: int):
         """Records price packet messages to log files.
 
         Args:
             account_id: Account id.
         """
-        prev_price = self._previousPrices[account_id][instance_index] if instance_index \
+        prev_price = self._previousPrices[account_id][instance_number] if instance_number \
             in self._previousPrices[account_id] else {'first': {}, 'last': {}}
         queue = self._writeQueue[account_id]['queue']
-        del self._previousPrices[account_id][instance_index]
+        del self._previousPrices[account_id][instance_number]
         if not len(self._previousPrices[account_id].keys()):
             del self._previousPrices[account_id]
         if prev_price['first']['sequenceNumber'] != prev_price['last']['sequenceNumber']:
             queue.append(json.dumps(prev_price['last']))
             queue.append(f'Recorded price packets {prev_price["first"]["sequenceNumber"]}'
-                         f'-{prev_price["last"]["sequenceNumber"]}, instanceIndex: {instance_index}')
+                         f'-{prev_price["last"]["sequenceNumber"]}, instanceIndex: {instance_number}')
 
     async def _append_logs(self):
         """Writes logs to files."""
@@ -207,7 +209,7 @@ class PacketLogger:
                     f.write(write_string)
                     f.close()
                 except Exception as err:
-                    print('Error writing log', err)
+                    print('Error writing log', format_error(err))
                 queue['isWriting'] = False
 
     async def _delete_old_data(self):
