@@ -8,21 +8,33 @@ from ..clients.metaApi.packetLogger import PacketLoggerOpts
 from ..clients.errorHandler import ValidationException
 from ..metaApi.connectionRegistry import ConnectionRegistry
 from .metatraderDemoAccountApi import MetatraderDemoAccountApi
+from ..clients.metaApi.synchronizationThrottler import SynchronizationThrottlerOpts
 from ..clients.metaApi.metatraderDemoAccount_client import MetatraderDemoAccountClient
+from ..clients.metaApi.historicalMarketData_client import HistoricalMarketDataClient
 from .latencyMonitor import LatencyMonitor
+from ..clients.metaApi.expertAdvisor_client import ExpertAdvisorClient
 import re
-import traceback
 from typing import Optional
+from ..metaApi.models import format_error
 from typing_extensions import TypedDict
 
 
 class RetryOpts(TypedDict):
+    """Request retry options."""
     retries: Optional[int]
     """Maximum amount of request retries, default value is 5."""
     minDelayInSeconds: Optional[float]
     """Minimum delay in seconds until request retry, default value is 1."""
     maxDelayInSeconds: Optional[float]
     """Maximum delay in seconds until request retry, default value is 30."""
+    subscribeCooldownInSeconds: Optional[float]
+    """Time to disable new subscriptions for """
+
+
+class EventProcessingOpts(TypedDict):
+    """Options for processing websocket client events."""
+    sequentialProcessing: Optional[bool]
+    """An option to process synchronization events after finishing previous ones."""
 
 
 class MetaApiOpts(TypedDict):
@@ -37,16 +49,24 @@ class MetaApiOpts(TypedDict):
     """Timeout for connecting to server in seconds."""
     packetOrderingTimeout: Optional[float]
     """Packet ordering timeout in seconds."""
+    historicalMarketDataRequestTimeout: Optional[float]
+    """Timeout for historical market data client in seconds."""
+    demoAccountRequestTimeout: Optional[float]
+    """Timeout for demo account requests in seconds."""
     packetLogger: Optional[PacketLoggerOpts]
     """Packet logger options."""
     enableLatencyMonitor: Optional[bool]
     """An option to enable latency tracking."""
     enableLatencyTracking: Optional[bool]
     """An option to enable latency tracking."""
-    maxConcurrentSynchronizations: Optional[int]
-    """Max concurrent synchronizations via websocket client."""
+    synchronizationThrottler: Optional[SynchronizationThrottlerOpts]
+    """Options for synchronization throttler."""
     retryOpts: Optional[RetryOpts]
     """Options for request retries."""
+    eventProcessing: Optional[EventProcessingOpts]
+    """Options for processing events."""
+    useSharedClientApi: Optional[bool]
+    """Option to use a shared server."""
 
 
 class MetaApi:
@@ -63,28 +83,39 @@ class MetaApi:
         application = opts['application'] if 'application' in opts else 'MetaApi'
         domain = opts['domain'] if 'domain' in opts else 'agiliumtrade.agiliumtrade.ai'
         request_timeout = opts['requestTimeout'] if 'requestTimeout' in opts else 60
+        historical_market_data_request_timeout = opts['historicalMarketDataRequestTimeout'] if \
+            'historicalMarketDataRequestTimeout' in opts else 240
         connect_timeout = opts['connectTimeout'] if 'connectTimeout' in opts else 60
         packet_ordering_timeout = opts['packetOrderingTimeout'] if 'packetOrderingTimeout' in opts else 60
         retry_opts = opts['retryOpts'] if 'retryOpts' in opts else {}
         packet_logger = opts['packetLogger'] if 'packetLogger' in opts else {}
-        max_concurrent_synchronizations = opts['maxConcurrentSynchronizations'] if 'maxConcurrentSynchronizations' in \
-                                                                                   opts else 5
+        synchronization_throttler = opts['synchronizationThrottler'] if 'synchronizationThrottler' in opts else {}
+        demo_account_request_timeout = opts['demoAccountRequestTimeout'] if \
+            'demoAccountRequestTimeout' in opts else 240
+        event_processing = opts['eventProcessing'] if 'eventProcessing' in opts else {}
+        use_shared_client_api = opts['useSharedClientApi'] if 'useSharedClientApi' in opts else False
         if not re.search(r"[a-zA-Z0-9_]+", application):
             raise ValidationException('Application name must be non-empty string consisting ' +
                                       'from letters, digits and _ only')
         http_client = HttpClient(request_timeout, retry_opts)
+        historical_market_data_http_client = HttpClient(historical_market_data_request_timeout, retry_opts)
+        demo_account_http_client = HttpClient(demo_account_request_timeout, retry_opts)
         self._metaApiWebsocketClient = MetaApiWebsocketClient(
-            token, {'application': application, 'domain': domain, 'requestTimeout': request_timeout,
-                    'connectTimeout': connect_timeout, 'packetLogger': packet_logger,
-                    'packetOrderingTimeout': packet_ordering_timeout,
-                    'maxConcurrentSynchronizations': max_concurrent_synchronizations,
-                    'retryOpts': retry_opts})
+            http_client, token, {'application': application, 'domain': domain, 'requestTimeout': request_timeout,
+                                 'connectTimeout': connect_timeout, 'packetLogger': packet_logger,
+                                 'packetOrderingTimeout': packet_ordering_timeout,
+                                 'synchronizationThrottler': synchronization_throttler,
+                                 'eventProcessing': event_processing, 'retryOpts': retry_opts,
+                                 'useSharedClientApi': use_shared_client_api})
         self._provisioningProfileApi = ProvisioningProfileApi(ProvisioningProfileClient(http_client, token, domain))
         self._connectionRegistry = ConnectionRegistry(self._metaApiWebsocketClient, application)
+        historical_market_data_client = HistoricalMarketDataClient(historical_market_data_http_client, token, domain)
         self._metatraderAccountApi = MetatraderAccountApi(MetatraderAccountClient(http_client, token, domain),
-                                                          self._metaApiWebsocketClient, self._connectionRegistry)
-        self._metatraderDemoAccountApi = MetatraderDemoAccountApi(MetatraderDemoAccountClient(http_client, token,
-                                                                                              domain))
+                                                          self._metaApiWebsocketClient, self._connectionRegistry,
+                                                          ExpertAdvisorClient(http_client, token, domain),
+                                                          historical_market_data_client)
+        self._metatraderDemoAccountApi = MetatraderDemoAccountApi(MetatraderDemoAccountClient(
+            demo_account_http_client, token, domain))
         if ('enableLatencyTracking' in opts and opts['enableLatencyTracking']) or ('enableLatencyMonitor' in opts and
                                                                                    opts['enableLatencyMonitor']):
             self._latencyMonitor = LatencyMonitor()
@@ -132,17 +163,7 @@ class MetaApi:
         Args:
             err: Exception to process.
         """
-        error = {'name': err.__class__.__name__, 'message': err.args[0]}
-        if hasattr(err, 'status_code'):
-            error['status_code'] = err.status_code
-        if err.__class__.__name__ == 'ValidationException':
-            error['details'] = err.details
-        if err.__class__.__name__ == 'TradeException':
-            error['string_code'] = err.stringCode
-        if err.__class__.__name__ == 'TooManyRequestsException':
-            error['metadata'] = err.metadata
-        error['trace'] = traceback.format_exc()
-        return error
+        return format_error(err)
 
     def close(self):
         """Closes all clients and connections"""
