@@ -8,9 +8,10 @@ from .metatraderAccountModel import MetatraderAccountModel
 from .connectionRegistryModel import ConnectionRegistryModel
 from .historyStorage import HistoryStorage
 from ..clients.timeoutException import TimeoutException
-from .models import random_id, MetatraderSymbolSpecification, MetatraderAccountInformation, \
+from .models import random_id, format_error, MetatraderSymbolSpecification, MetatraderAccountInformation, \
     MetatraderPosition, MetatraderOrder, MetatraderHistoryOrders, MetatraderDeals, MetatraderTradeResponse, \
-    MetatraderSymbolPrice, MarketTradeOptions, PendingTradeOptions
+    MetatraderSymbolPrice, MarketTradeOptions, PendingTradeOptions, MarketDataSubscription, MarketDataUnsubscription, \
+    MetatraderCandle, MetatraderTick, MetatraderBook, StopOptions
 from datetime import datetime, timedelta
 from typing import Coroutine, List, Optional, Dict
 from typing_extensions import TypedDict
@@ -73,14 +74,10 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         self._websocketClient.add_synchronization_listener(account.id, self._terminalState)
         self._websocketClient.add_synchronization_listener(account.id, self._historyStorage)
         self._websocketClient.add_synchronization_listener(account.id, self._healthMonitor)
-        self._websocketClient.add_reconnect_listener(self)
+        self._websocketClient.add_reconnect_listener(self, account.id)
         self._subscriptions = {}
         self._stateByInstanceIndex = {}
         self._synchronized = False
-        self._shouldRetrySubscribe = False
-        self._isSubscribing = False
-        self._subscribeTask = None
-        self._subscribeFuture = None
 
     def get_account_information(self) -> 'Coroutine[asyncio.Future[MetatraderAccountInformation]]':
         """Returns account information (see
@@ -237,7 +234,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         asyncio.create_task(self._historyStorage.clear())
         return self._websocketClient.remove_application(self._account.id)
 
-    def create_market_buy_order(self, symbol: str, volume: float, stop_loss: float = None, take_profit: float = None,
+    def create_market_buy_order(self, symbol: str, volume: float, stop_loss: float or StopOptions = None,
+                                take_profit: float or StopOptions = None,
                                 options: MarketTradeOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a market buy order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
@@ -255,15 +253,13 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Raises:
             TradeException: On trade error, check error properties for error code details.
         """
-        trade_params = {'actionType': 'ORDER_TYPE_BUY', 'symbol': symbol, 'volume': volume}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+        trade_params = {'actionType': 'ORDER_TYPE_BUY', 'symbol': symbol, 'volume': volume,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def create_market_sell_order(self, symbol: str, volume: float, stop_loss: float = None, take_profit: float = None,
+    def create_market_sell_order(self, symbol: str, volume: float, stop_loss: float or StopOptions = None,
+                                 take_profit: float or StopOptions = None,
                                  options: MarketTradeOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a market sell order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
@@ -281,16 +277,14 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Raises:
             TradeException: On trade error, check error properties for error code details.
         """
-        trade_params = {'actionType': 'ORDER_TYPE_SELL', 'symbol': symbol, 'volume': volume}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+        trade_params = {'actionType': 'ORDER_TYPE_SELL', 'symbol': symbol, 'volume': volume,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def create_limit_buy_order(self, symbol: str, volume: float, open_price: float, stop_loss: float = None,
-                               take_profit: float = None, options: PendingTradeOptions = None) -> \
+    def create_limit_buy_order(self, symbol: str, volume: float, open_price: float,
+                               stop_loss: float or StopOptions = None,
+                               take_profit: float or StopOptions = None, options: PendingTradeOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a limit buy order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -309,16 +303,14 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_BUY_LIMIT', 'symbol': symbol, 'volume': volume,
-                        'openPrice': open_price}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+                        'openPrice': open_price,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def create_limit_sell_order(self, symbol: str, volume: float, open_price: float, stop_loss: float = None,
-                                take_profit: float = None, options: PendingTradeOptions = None) -> \
+    def create_limit_sell_order(self, symbol: str, volume: float, open_price: float,
+                                stop_loss: float or StopOptions = None,
+                                take_profit: float or StopOptions = None, options: PendingTradeOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a limit sell order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -337,16 +329,14 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_SELL_LIMIT', 'symbol': symbol, 'volume': volume,
-                        'openPrice': open_price}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+                        'openPrice': open_price,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def create_stop_buy_order(self, symbol: str, volume: float, open_price: float, stop_loss: float = None,
-                              take_profit: float = None, options: PendingTradeOptions = None) -> \
+    def create_stop_buy_order(self, symbol: str, volume: float, open_price: float,
+                              stop_loss: float or StopOptions = None,
+                              take_profit: float or StopOptions = None, options: PendingTradeOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a stop buy order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -365,16 +355,14 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_BUY_STOP', 'symbol': symbol, 'volume': volume,
-                        'openPrice': open_price}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+                        'openPrice': open_price,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def create_stop_sell_order(self, symbol: str, volume: float, open_price: float, stop_loss: float = None,
-                               take_profit: float = None, options: PendingTradeOptions = None) -> \
+    def create_stop_sell_order(self, symbol: str, volume: float, open_price: float,
+                               stop_loss: float or StopOptions = None,
+                               take_profit: float or StopOptions = None, options: PendingTradeOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a stop sell order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -393,16 +381,13 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_SELL_STOP', 'symbol': symbol, 'volume': volume,
-                        'openPrice': open_price}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+                        'openPrice': open_price,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def create_stop_limit_buy_order(self, symbol: str, volume: float, open_price: float, stop_limit_price: float,
-                                    stop_loss: float = None, take_profit: float = None,
+                                    stop_loss: float or StopOptions = None, take_profit: float or StopOptions = None,
                                     options: PendingTradeOptions = None):
         """Creates a stop limit buy order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -422,16 +407,13 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_BUY_STOP_LIMIT', 'symbol': symbol, 'volume': volume,
-                        'openPrice': open_price, 'stopLimitPrice': stop_limit_price}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+                        'openPrice': open_price, 'stopLimitPrice': stop_limit_price,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def create_stop_limit_sell_order(self, symbol: str, volume: float, open_price: float, stop_limit_price: float,
-                                     stop_loss: float = None, take_profit: float = None,
+                                     stop_loss: float or StopOptions = None, take_profit: float or StopOptions = None,
                                      options: PendingTradeOptions = None):
         """Creates a stop limit sell order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -451,15 +433,13 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_SELL_STOP_LIMIT', 'symbol': symbol, 'volume': volume,
-                        'openPrice': open_price, 'stopLimitPrice': stop_limit_price}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+                        'openPrice': open_price, 'stopLimitPrice': stop_limit_price,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def modify_position(self, position_id: str, stop_loss: float = None, take_profit: float = None) -> \
+    def modify_position(self, position_id: str, stop_loss: float or StopOptions = None,
+                        take_profit: float or StopOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Modifies a position (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -474,11 +454,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Raises:
             TradeException: On trade error.
         """
-        trade_params = {'actionType': 'POSITION_MODIFY', 'positionId': position_id}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+        trade_params = {'actionType': 'POSITION_MODIFY', 'positionId': position_id,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def close_position_partially(self, position_id: str, volume: float, options: MarketTradeOptions = None) -> \
@@ -556,8 +533,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def modify_order(self, order_id: str, open_price: float, stop_loss: float = None,
-                     take_profit: float = None) -> \
+    def modify_order(self, order_id: str, open_price: float, stop_loss: float or StopOptions = None,
+                     take_profit: float or StopOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Modifies a pending order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -573,11 +550,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Raises:
             TradeException: On trade error, check error properties for error code details.
         """
-        trade_params = {'actionType': 'ORDER_MODIFY', 'orderId': order_id, 'openPrice': open_price}
-        if stop_loss:
-            trade_params['stopLoss'] = stop_loss
-        if take_profit:
-            trade_params['takeProfit'] = take_profit
+        trade_params = {'actionType': 'ORDER_MODIFY', 'orderId': order_id, 'openPrice': open_price,
+                        **self._generate_stop_options(stop_loss=stop_loss, take_profit=take_profit)}
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def cancel_order(self, order_id: str) -> \
@@ -603,7 +577,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.reconnect(self._account.id)
 
-    async def synchronize(self, instance_index: int) -> Coroutine:
+    async def synchronize(self, instance_index: str) -> Coroutine:
         """Requests the terminal to start synchronization process.
         (see https://metaapi.cloud/docs/client/websocket/synchronizing/synchronize/).
 
@@ -613,17 +587,19 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine which resolves when synchronization started.
         """
+        instance = self.get_instance_number(instance_index)
+        host = self.get_host_name(instance_index)
         starting_history_order_time = \
             datetime.utcfromtimestamp(max(((self._history_start_time and self._history_start_time.timestamp()) or 0),
-                                      (await self._historyStorage.last_history_order_time(instance_index))
+                                      (await self._historyStorage.last_history_order_time(instance))
                                           .timestamp())).replace(tzinfo=pytz.UTC)
         starting_deal_time = \
             datetime.utcfromtimestamp(max(((self._history_start_time and self._history_start_time.timestamp()) or 0),
-                                      (await self._historyStorage.last_deal_time(instance_index)).timestamp()))\
+                                      (await self._historyStorage.last_deal_time(instance)).timestamp()))\
             .replace(tzinfo=pytz.UTC)
         synchronization_id = random_id()
         self._get_state(instance_index)['lastSynchronizationId'] = synchronization_id
-        return await self._websocketClient.synchronize(self._account.id, instance_index, synchronization_id,
+        return await self._websocketClient.synchronize(self._account.id, instance, host, synchronization_id,
                                                        starting_history_order_time, starting_deal_time)
 
     async def initialize(self):
@@ -636,60 +612,80 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine which resolves when subscription is initiated.
         """
-        if not self._isSubscribing:
-            self._isSubscribing = True
-            self._shouldRetrySubscribe = True
-            subscribe_retry_interval_in_seconds = 3
-            while self._shouldRetrySubscribe and not self._closed:
-                try:
-                    await self._websocketClient.subscribe(self._account.id)
-                except Exception:
-                    pass
-                if not self._shouldRetrySubscribe:
-                    break
-                retry_interval = subscribe_retry_interval_in_seconds
-                subscribe_retry_interval_in_seconds = min(subscribe_retry_interval_in_seconds * 2, 300)
-                subscribe_future = asyncio.Future()
+        if not self._closed:
+            self._websocketClient.ensure_subscribe(self._account.id)
 
-                async def subscribe_task():
-                    await asyncio.sleep(retry_interval)
-                    subscribe_future.set_result(True)
-                self._subscribeTask = asyncio.create_task(subscribe_task())
-                self._subscribeFuture = subscribe_future
-                result = await self._subscribeFuture
-                self._subscribeFuture = None
-                if not result:
-                    break
-            self._isSubscribing = False
-
-    def subscribe_to_market_data(self, symbol: str, instance_index: int = None) -> Coroutine:
+    def subscribe_to_market_data(self, symbol: str, subscriptions: List[MarketDataSubscription] = None,
+                                 instance_index: int = None) -> Coroutine:
         """Subscribes on market data of specified symbol (see
         https://metaapi.cloud/docs/client/websocket/marketDataStreaming/subscribeToMarketData/).
 
         Args:
             symbol: Symbol (e.g. currency pair or an index).
+            subscriptions: Array of market data subscription to create or update. Please note that this feature is
+            not fully implemented on server-side yet.
             instance_index: Instance index.
 
         Returns:
             Promise which resolves when subscription request was processed.
         """
-        self._subscriptions[symbol] = True
-        return self._websocketClient.subscribe_to_market_data(self._account.id, instance_index, symbol)
+        self._subscriptions[symbol] = {'subscriptions': subscriptions or None}
+        return self._websocketClient.subscribe_to_market_data(self._account.id, instance_index, symbol,
+                                                              subscriptions)
 
-    def unsubscribe_from_market_data(self, symbol: str, instance_index: int = None) -> Coroutine:
+    def unsubscribe_from_market_data(self, symbol: str, subscriptions: List[MarketDataUnsubscription] = None,
+                                     instance_index: int = None) -> Coroutine:
         """Unsubscribes from market data of specified symbol (see
         https://metaapi.cloud/docs/client/websocket/marketDataStreaming/subscribeToMarketData/).
 
         Args:
             symbol: Symbol (e.g. currency pair or an index).
+            subscriptions: Array of subscriptions to cancel.
             instance_index: Instance index.
 
         Returns:
             Promise which resolves when subscription request was processed.
         """
-        if symbol in self._subscriptions:
+        if not subscriptions:
+            if symbol in self._subscriptions:
+                del self._subscriptions[symbol]
+        elif symbol in self._subscriptions:
+            self._subscriptions[symbol]['subscriptions'] = list(filter(
+                lambda s: not next((s2 for s2 in subscriptions if s['type'] == s2['type']), None),
+                self._subscriptions[symbol]['subscriptions']))
+            if not len(self._subscriptions[symbol]['subscriptions']):
+                del self._subscriptions[symbol]
+        return self._websocketClient.unsubscribe_from_market_data(self._account.id, instance_index, symbol,
+                                                                  subscriptions)
+
+    async def on_subscription_downgraded(self, instance_index: str, symbol: str,
+                                         updates: List[MarketDataSubscription] or None = None,
+                                         unsubscriptions: List[MarketDataUnsubscription] or None = None):
+        """Invoked when subscription downgrade has occurred.
+
+        Args:
+            instance_index: Index of an account instance connected.
+            symbol: Symbol to update subscriptions for.
+            updates: Array of market data subscription to update.
+            unsubscriptions: Array of subscriptions to cancel.
+
+        Returns:
+            A coroutine which resolves when the asynchronous event is processed.
+        """
+        subscriptions = self._subscriptions[symbol] if symbol in self._subscriptions else None
+        if unsubscriptions and len(unsubscriptions):
+            if subscriptions:
+                for subscription in unsubscriptions:
+                    subscriptions = list(filter(lambda s: s['type'] == subscription['type'], subscriptions))
+            asyncio.create_task(self.unsubscribe_from_market_data(symbol, unsubscriptions))
+        if updates and len(updates):
+            if subscriptions:
+                for subscription in updates:
+                    for s in list(filter(lambda s: s['type'] == subscription['type'], subscriptions)):
+                        s['intervalInMilliiseconds'] = subscription['intervalInMilliseconds']
+            asyncio.create_task(self.subscribe_to_market_data(symbol, updates))
+        if subscriptions and (not len(subscriptions)):
             del self._subscriptions[symbol]
-        return self._websocketClient.unsubscribe_from_market_data(self._account.id, instance_index, symbol)
 
     @property
     def subscribed_symbols(self) -> List[str]:
@@ -700,9 +696,29 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return list(self._subscriptions.keys())
 
+    def subscriptions(self, symbol) -> List[MarketDataSubscription]:
+        """Returns subscriptions for a symbol.
+
+        Args:
+            symbol: Symbol to retrieve subscriptions for.
+
+        Returns:
+            List of market data subscriptions for the symbol.
+        """
+        return self._subscriptions[symbol]['subscriptions'] if symbol in self._subscriptions else []
+
+    def get_symbols(self) -> 'Coroutine[asyncio.Future[List[str]]]':
+        """Retrieves available symbols for an account (see
+        https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/readSymbols/).
+
+        Returns:
+            A coroutine which resolves when symbols are retrieved.
+        """
+        return self._websocketClient.get_symbols(self._account.id)
+
     def get_symbol_specification(self, symbol: str) -> 'Coroutine[asyncio.Future[MetatraderSymbolSpecification]]':
         """Retrieves specification for a symbol (see
-        https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/getSymbolSpecification/).
+        https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/readSymbolSpecification/).
 
         Args:
             symbol: Symbol to retrieve specification for.
@@ -713,8 +729,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         return self._websocketClient.get_symbol_specification(self._account.id, symbol)
 
     def get_symbol_price(self, symbol) -> 'Coroutine[asyncio.Future[MetatraderSymbolPrice]]':
-        """Retrieves specification for a symbol (see
-        https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/getSymbolPrice/).
+        """Retrieves latest price for a symbol (see
+        https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/readSymbolPrice/).
 
         Args:
             symbol: Symbol to retrieve price for.
@@ -724,6 +740,45 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_symbol_price(self._account.id, symbol)
 
+    def get_candle(self, symbol: str, timeframe: str) -> 'Coroutine[asyncio.Future[MetatraderCandle]]':
+        """Retrieves latest candle for a symbol and timeframe (see
+        https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/readCandle/).
+
+        Args:
+            symbol: Symbol to retrieve candle for.
+            timeframe: Defines the timeframe according to which the candle must be generated. Allowed values for
+            MT5 are 1m, 2m, 3m, 4m, 5m, 6m, 10m, 12m, 15m, 20m, 30m, 1h, 2h, 3h, 4h, 6h, 8h, 12h, 1d, 1w, 1mn.
+            Allowed values for MT4 are 1m, 5m, 15m 30m, 1h, 4h, 1d, 1w, 1mn.
+
+        Returns:
+            A coroutine which resolves when candle is retrieved.
+        """
+        return self._websocketClient.get_candle(self._account.id, symbol, timeframe)
+
+    def get_tick(self, symbol: str) -> 'Coroutine[asyncio.Future[MetatraderTick]]':
+        """Retrieves latest tick for a symbol (see
+        https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/readTick/).
+
+        Args:
+            symbol: Symbol to retrieve tick for.
+
+        Returns:
+            A coroutine which resolves when tick is retrieved.
+        """
+        return self._websocketClient.get_tick(self._account.id, symbol)
+
+    def get_book(self, symbol: str) -> 'Coroutine[asyncio.Future[MetatraderBook]]':
+        """Retrieves latest order book for a symbol (see
+        https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/readBook/).
+
+        Args:
+            symbol: Symbol to retrieve order book for.
+
+        Returns:
+            A coroutine which resolves when order book is retrieved.
+        """
+        return self._websocketClient.get_book(self._account.id, symbol)
+
     def save_uptime(self, uptime: Dict):
         """Sends client uptime stats to the server.
 
@@ -769,7 +824,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         self._websocketClient.remove_synchronization_listener(self._account.id, listener)
 
-    async def on_connected(self, instance_index: int, replicas: int):
+    async def on_connected(self, instance_index: str, replicas: int):
         """Invoked when connection to MetaTrader terminal established.
 
         Args:
@@ -779,25 +834,21 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine which resolves when the asynchronous event is processed.
         """
-        if self._subscribeFuture and not self._subscribeFuture.done():
-            self._subscribeFuture.set_result(False)
-            self._subscribeTask.cancel()
-        self._shouldRetrySubscribe = False
         key = random_id(32)
         state = self._get_state(instance_index)
         state['shouldSynchronize'] = key
         state['synchronizationRetryIntervalInSeconds'] = 1
         state['synchronized'] = False
-        await self._ensure_synchronized(instance_index, key)
+        asyncio.create_task(self._ensure_synchronized(instance_index, key))
         indices = []
         for i in range(replicas):
             indices.append(i)
         for key in list(self._stateByInstanceIndex.keys()):
             e = self._stateByInstanceIndex[key]
-            if e['instanceIndex'] not in indices:
+            if self.get_instance_number(e['instanceIndex']) not in indices:
                 del self._stateByInstanceIndex[key]
 
-    async def on_disconnected(self, instance_index: int):
+    async def on_disconnected(self, instance_index: str):
         """Invoked when connection to MetaTrader terminal terminated.
 
         Args:
@@ -813,7 +864,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         state['synchronized'] = False
         state['disconnected'] = True
 
-    async def on_deal_synchronization_finished(self, instance_index: int, synchronization_id: str):
+    async def on_deal_synchronization_finished(self, instance_index: str, synchronization_id: str):
         """Invoked when a synchronization of history deals on a MetaTrader account have finished.
 
         Args:
@@ -826,7 +877,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         state = self._get_state(instance_index)
         state['dealsSynchronized'][synchronization_id] = True
 
-    async def on_order_synchronization_finished(self, instance_index: int, synchronization_id: str):
+    async def on_order_synchronization_finished(self, instance_index: str, synchronization_id: str):
         """Invoked when a synchronization of history orders on a MetaTrader account have finished.
 
         Args:
@@ -839,18 +890,51 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         state = self._get_state(instance_index)
         state['ordersSynchronized'][synchronization_id] = True
 
+    async def on_account_information_updated(self, instance_index: str,
+                                             account_information: MetatraderAccountInformation):
+        """Invoked when MetaTrader position is updated.
+
+        Args:
+            instance_index: Index of an account instance connected.
+            account_information: Updated MetaTrader position.
+
+        Returns:
+            A coroutine which resolves when the asynchronous event is processed.
+        """
+        async def subscribe_task(symbol):
+            try:
+                instance = self.get_instance_number(instance_index)
+                await self.subscribe_to_market_data(symbol, self._subscriptions[symbol]['subscriptions'],
+                                                    instance)
+            except Exception as err:
+                print(f'[{datetime.now().isoformat()}] MetaApi websocket client for account ' + self.account.id +
+                      ':' + str(instance_index) + ' failed to resubscribe to symbol ' + symbol, format_error(err))
+
+        for symbol in self.subscribed_symbols:
+            if not self._terminalState.price(symbol):
+                asyncio.create_task(subscribe_task(symbol))
+
     async def on_reconnected(self):
         """Invoked when connection to MetaApi websocket API restored after a disconnect.
 
         Returns:
             A coroutine which resolves when connection to MetaApi websocket API restored after a disconnect.
         """
-        if self._subscribeFuture and not self._subscribeFuture.done():
-            self._subscribeFuture.set_result(False)
-            self._subscribeTask.cancel()
-        await self.subscribe()
+        self._stateByInstanceIndex = {}
 
-    async def is_synchronized(self, instance_index: int, synchronization_id: str = None) -> bool:
+    async def on_stream_closed(self, instance_index: str):
+        """Invoked when a stream for an instance index is closed.
+
+        Args:
+            instance_index: Index of an account instance connected.
+
+        Returns:
+            A coroutine which resolves when the asynchronous event is processed.
+        """
+        if instance_index in self._stateByInstanceIndex:
+            del self._stateByInstanceIndex[instance_index]
+
+    async def is_synchronized(self, instance_index: str, synchronization_id: str = None) -> bool:
         """Returns flag indicating status of state synchronization with MetaTrader terminal.
 
         Args:
@@ -870,7 +954,6 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
                 synchronization_id in s['dealsSynchronized'] and \
                 bool(s['dealsSynchronized'][synchronization_id])
             return acc or synchronized
-
         return reduce(reducer_func, self._stateByInstanceIndex.values(), False) if \
             len(self._stateByInstanceIndex.values()) else False
 
@@ -912,8 +995,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
                                    (synchronization_id or (bool(state) and state['lastSynchronizationId']) or
                                     (bool(state) and state['lastDisconnectedSynchronizationId']) or 'None'))
         time_left_in_seconds = max(0, timeout_in_seconds - (datetime.now() - start_time).total_seconds())
-        await self._websocketClient.wait_synchronized(self._account.id, instance_index, application_pattern,
-                                                      time_left_in_seconds)
+        await self._websocketClient.wait_synchronized(self._account.id, self.get_instance_number(instance_index),
+                                                      application_pattern, time_left_in_seconds)
 
     async def close(self):
         """Closes the connection. The instance of the class should no longer be used after this method is invoked."""
@@ -924,6 +1007,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             self._websocketClient.remove_synchronization_listener(self._account.id, self._terminalState)
             self._websocketClient.remove_synchronization_listener(self._account.id, self._historyStorage)
             self._websocketClient.remove_synchronization_listener(self._account.id, self._healthMonitor)
+            self._websocketClient.remove_reconnect_listener(self)
             self._connection_registry.remove(self._account.id)
             self._healthMonitor.stop()
             self._closed = True
@@ -955,18 +1039,31 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._healthMonitor
 
-    async def _ensure_synchronized(self, instance_index: int, key):
+    def _generate_stop_options(self, stop_loss, take_profit):
+        trade = {}
+        if isinstance(stop_loss, int) or isinstance(stop_loss, float):
+            trade['stopLoss'] = stop_loss
+        elif stop_loss:
+            trade['stopLoss'] = stop_loss['value']
+            trade['stopLossUnits'] = stop_loss['units']
+        if isinstance(take_profit, int) or isinstance(take_profit, float):
+            trade['takeProfit'] = take_profit
+        elif take_profit:
+            trade['takeProfit'] = take_profit['value']
+            trade['takeProfitUnits'] = take_profit['units']
+        return trade
+
+    async def _ensure_synchronized(self, instance_index: str, key):
         state = self._get_state(instance_index)
-        if state:
+        if state and not self._closed:
             try:
-                await self.synchronize(instance_index)
-                for symbol in self.subscribed_symbols:
-                    await self.subscribe_to_market_data(symbol, instance_index)
-                state['synchronized'] = True
-                state['synchronizationRetryIntervalInSeconds'] = 1
+                synchronization_result = await self.synchronize(instance_index)
+                if synchronization_result:
+                    state['synchronized'] = True
+                    state['synchronizationRetryIntervalInSeconds'] = 1
             except Exception as err:
                 print(f'[{datetime.now().isoformat()}] MetaApi websocket client for account ' + self.account.id +
-                      ':' + str(instance_index) + ' failed to synchronize', err)
+                      ':' + str(instance_index) + ' failed to synchronize', format_error(err))
                 if state['shouldSynchronize'] == key:
 
                     async def restart_ensure_sync():
@@ -976,9 +1073,9 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
                     state['synchronizationRetryIntervalInSeconds'] = \
                         min(state['synchronizationRetryIntervalInSeconds'] * 2, 300)
 
-    def _get_state(self, instance_index: int) -> MetaApiConnectionDict:
-        if str(instance_index) not in self._stateByInstanceIndex:
-            self._stateByInstanceIndex[str(instance_index)] = {
+    def _get_state(self, instance_index: str) -> MetaApiConnectionDict:
+        if instance_index not in self._stateByInstanceIndex:
+            self._stateByInstanceIndex[instance_index] = {
                 'instanceIndex': instance_index,
                 'ordersSynchronized': {},
                 'dealsSynchronized': {},
@@ -989,4 +1086,4 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
                 'lastSynchronizationId': None,
                 'disconnected': False
             }
-        return self._stateByInstanceIndex[str(instance_index)]
+        return self._stateByInstanceIndex[instance_index]
