@@ -3,7 +3,6 @@ from ..clients.metaApi.metaApiWebsocket_client import MetaApiWebsocketClient
 from .models import MetatraderHistoryOrders, MetatraderDeals
 from ..clients.metaApi.reconnectListener import ReconnectListener
 from ..clients.metaApi.synchronizationListener import SynchronizationListener
-from ..clients.timeoutException import TimeoutException
 from .metatraderAccount import MetatraderAccount
 from datetime import datetime, timedelta
 from mock import MagicMock, AsyncMock, patch
@@ -59,23 +58,23 @@ class MockClient(MetaApiWebsocketClient):
     def reconnect(self, account_id: str):
         pass
 
-    def synchronize(self, account_id: str, instance_index: int, synchronization_id: str,
+    def synchronize(self, account_id: str, instance_index: str, synchronization_id: str,
                     starting_history_order_time: datetime, starting_deal_time: datetime) -> Coroutine:
         pass
 
-    def subscribe(self, account_id: str, instance_index: int = None):
+    def subscribe(self, account_id: str, instance_index: str = None):
         pass
 
-    def subscribe_to_market_data(self, account_id: str, instance_index: int, symbol: str) -> Coroutine:
+    def subscribe_to_market_data(self, account_id: str, instance_index: str, symbol: str) -> Coroutine:
         pass
 
-    def unsubscribe_from_market_data(self, account_id: str, instance_index: int, symbol: str) -> Coroutine:
+    def unsubscribe_from_market_data(self, account_id: str, instance_index: str, symbol: str) -> Coroutine:
         pass
 
     def add_synchronization_listener(self, account_id: str, listener):
         pass
 
-    def add_reconnect_listener(self, listener: ReconnectListener):
+    def add_reconnect_listener(self, listener: ReconnectListener, account_id: str):
         pass
 
     def remove_synchronization_listener(self, account_id: str, listener: SynchronizationListener):
@@ -87,12 +86,19 @@ class MockClient(MetaApiWebsocketClient):
     def get_symbol_price(self, account_id: str, symbol: str) -> asyncio.Future:
         pass
 
-    async def wait_synchronized(self, account_id: str, instance_index: int, application_pattern: str,
+    async def wait_synchronized(self, account_id: str, instance_index: str, application_pattern: str,
                                 timeout_in_seconds: float):
         pass
 
 
 class MockAccount(MetatraderAccount):
+
+    def __init__(self, data, metatrader_account_client,
+                 meta_api_websocket_client, connection_registry):
+        super(MockAccount, self).__init__(data, metatrader_account_client, meta_api_websocket_client,
+                                          connection_registry, MagicMock(), MagicMock())
+        self._state = 'DEPLOYED'
+
     @property
     def id(self):
         return 'accountId'
@@ -101,6 +107,13 @@ class MockAccount(MetatraderAccount):
     def synchronization_mode(self):
         return 'user'
 
+    @property
+    def state(self):
+        return self._state
+
+    async def reload(self):
+        pass
+
 
 class AutoMockAccount(MetatraderAccount):
     @property
@@ -123,9 +136,9 @@ async def run_around_tests():
     global account
     account = MockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock())
     global auto_account
-    auto_account = AutoMockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock())
+    auto_account = AutoMockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock(), MagicMock(), MagicMock())
     global client
-    client = MockClient('token')
+    client = MockClient(MagicMock(), 'token')
     storage = MagicMock()
     storage.last_history_order_time = AsyncMock(return_value=datetime.now())
     storage.last_deal_time = AsyncMock(return_value=datetime.now())
@@ -133,7 +146,6 @@ async def run_around_tests():
     api = MetaApiConnection(client, account, storage, MagicMock())
     yield
     api.health_monitor.stop()
-    client._synchronizationThrottler.stop()
 
 
 class TestMetaApiConnection:
@@ -445,6 +457,25 @@ class TestMetaApiConnection:
                                                       'volume': 0.07, 'stopLoss': 0.9, 'takeProfit': 2.0,
                                                       'comment': 'comment', 'clientId': 'TE_GBPUSD_7hyINWqAlE'})
 
+    @pytest.mark.asyncio
+    async def test_create_market_buy_order_with_relative_sl_tp(self):
+        """Should create market buy order with relative SL/TP."""
+        trade_result = {
+            'error': 10009,
+            'description': 'TRADE_RETCODE_DONE',
+            'orderId': 46870472
+        }
+        client.trade = AsyncMock(return_value=trade_result)
+        actual = await api.create_market_buy_order('GBPUSD', 0.07, {'value': 0.1, 'units': 'RELATIVE_PRICE'},
+                                                   {'value': 2000, 'units': 'RELATIVE_POINTS'},
+                                                   {'comment': 'comment', 'clientId': 'TE_GBPUSD_7hyINWqAlE'})
+        assert actual == trade_result
+        client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_BUY', 'symbol': 'GBPUSD',
+                                                      'volume': 0.07, 'stopLoss': 0.1,
+                                                      'stopLossUnits': 'RELATIVE_PRICE', 'takeProfit': 2000,
+                                                      'takeProfitUnits': 'RELATIVE_POINTS', 'comment': 'comment',
+                                                      'clientId': 'TE_GBPUSD_7hyINWqAlE'})
+
     @pytest.mark.asyncio
     async def test_create_market_sell_order(self):
         """Should create market sell order."""
@@ -670,72 +701,18 @@ class TestMetaApiConnection:
     @pytest.mark.asyncio
     async def test_subscribe_to_terminal(self):
         """Should subscribe to terminal."""
-        client.subscribe = AsyncMock()
-
-        async def delay_connect():
-            await asyncio.sleep(0.1)
-            await api.on_connected(1, 1)
-
-        asyncio.create_task(delay_connect())
+        client.ensure_subscribe = AsyncMock()
         await api.subscribe()
-        client.subscribe.assert_called_with('accountId')
+        client.ensure_subscribe.assert_called_with('accountId')
 
     @pytest.mark.asyncio
-    async def test_retry_subscribe_to_terminal(self):
-        """Should retry subscribe to terminal if no response received."""
-        with patch('lib.metaApi.metaApiConnection.asyncio.sleep', new=lambda x: sleep(x / 10)):
-            response = {'type': 'response', 'accountId': 'accountId', 'requestId': 'requestId'}
-            client.subscribe = AsyncMock(side_effect=[TimeoutException('timeout'), response, response])
-
-            async def delay_connect():
-                await sleep(0.36)
-                await api.on_connected(1, 1)
-
-            asyncio.create_task(delay_connect())
-            await api.subscribe()
-            client.subscribe.assert_called_with('accountId')
-            assert client.subscribe.call_count == 2
-
-    @pytest.mark.asyncio
-    async def test_subscribe_after_reconnect(self):
-        """Should subscribe after reconnect."""
-        with patch('lib.metaApi.metaApiConnection.asyncio.sleep', new=lambda x: sleep(x / 10)):
-            client.connect = AsyncMock()
-            client.subscribe = AsyncMock()
-            asyncio.create_task(api.subscribe())
-            await sleep(0.18)
-            client.subscribe.assert_called_with('accountId')
-            await api.on_connected(1, 1)
-            asyncio.create_task(api.on_reconnected())
-            await sleep(0.33)
-            assert client.subscribe.call_count == 3
-            await api.on_connected(1, 1)
-            await sleep(0.18)
-            assert client.subscribe.call_count == 3
-
-    @pytest.mark.asyncio
-    async def test_no_multiple_subscribes(self):
-        """Should not send multiple subscribe requests at the same time."""
-        client.subscribe = AsyncMock()
-        asyncio.create_task(api.subscribe())
-        asyncio.create_task(api.subscribe())
-        await asyncio.sleep(0.5)
-        await api.on_connected(1, 1)
-        await asyncio.sleep(0.6)
-        client.subscribe.assert_called_with('accountId')
-        assert client.subscribe.call_count == 1
-
-    @pytest.mark.asyncio
-    async def test_not_retry_subscribe_after_close(self):
-        """Should not retry subscribe to terminal if connection is closed."""
-        client.subscribe = AsyncMock(side_effect=TimeoutException('timeout'))
+    async def test_not_subscribe_if_closed(self):
+        """Should not subscribe if connection is closed."""
+        client.ensure_subscribe = AsyncMock()
         client.unsubscribe = AsyncMock()
-        asyncio.create_task(api.subscribe())
-        asyncio.create_task(api.close())
-        await asyncio.sleep(1)
-        client.subscribe.assert_called_with('accountId')
-        assert client.subscribe.call_count == 1
-        client.unsubscribe.assert_called_with('accountId')
+        await api.close()
+        await api.subscribe()
+        client.ensure_subscribe.assert_not_called()
 
     @pytest.mark.asyncio
     async def test_synchronize_state_with_terminal(self):
@@ -743,11 +720,12 @@ class TestMetaApiConnection:
         client.synchronize = AsyncMock()
         with patch('lib.metaApi.metaApiConnection.random_id', return_value='synchronizationId'):
             api = MetaApiConnection(client, account, None, MagicMock())
-            await api.history_storage.on_history_order_added(1, {'doneTime': date('2020-01-01T00:00:00.000Z')})
-            await api.history_storage.on_deal_added(1, {'time': date('2020-01-02T00:00:00.000Z')})
-            await api.synchronize(1)
-            client.synchronize.assert_called_with('accountId', 1, 'synchronizationId', date('2020-01-01T00:00:00.000Z'),
-                                                  date('2020-01-02T00:00:00.000Z'))
+            await api.history_storage.on_history_order_added('1:ps-mpa-1',
+                                                             {'doneTime': date('2020-01-01T00:00:00.000Z')})
+            await api.history_storage.on_deal_added('1:ps-mpa-1', {'time': date('2020-01-02T00:00:00.000Z')})
+            await api.synchronize('1:ps-mpa-1')
+            client.synchronize.assert_called_with('accountId', 1, 'ps-mpa-1', 'synchronizationId',
+                                                  date('2020-01-01T00:00:00.000Z'), date('2020-01-02T00:00:00.000Z'))
 
     @pytest.mark.asyncio
     async def test_synchronize_state_with_terminal_from_time(self):
@@ -755,30 +733,59 @@ class TestMetaApiConnection:
         client.synchronize = AsyncMock()
         with patch('lib.metaApi.metaApiConnection.random_id', return_value='synchronizationId'):
             api = MetaApiConnection(client, account, None, MagicMock(), date('2020-10-07T00:00:00.000Z'))
-            await api.history_storage.on_history_order_added(1, {'doneTime': date('2020-01-01T00:00:00.000Z')})
-            await api.history_storage.on_deal_added(1, {'time': date('2020-01-02T00:00:00.000Z')})
-            await api.synchronize(1)
-            client.synchronize.assert_called_with('accountId', 1, 'synchronizationId', date('2020-10-07T00:00:00.000Z'),
-                                                  date('2020-10-07T00:00:00.000Z'))
+            await api.history_storage.on_history_order_added('1:ps-mpa-1',
+                                                             {'doneTime': date('2020-01-01T00:00:00.000Z')})
+            await api.history_storage.on_deal_added('1:ps-mpa-1', {'time': date('2020-01-02T00:00:00.000Z')})
+            await api.synchronize('1:ps-mpa-1')
+            client.synchronize.assert_called_with('accountId', 1, 'ps-mpa-1', 'synchronizationId',
+                                                  date('2020-10-07T00:00:00.000Z'), date('2020-10-07T00:00:00.000Z'))
 
     @pytest.mark.asyncio
     async def test_subscribe_to_market_data(self):
         """Should subscribe to market data."""
         client.subscribe_to_market_data = AsyncMock()
-        await api.subscribe_to_market_data('EURUSD', 1)
+        await api.subscribe_to_market_data('EURUSD', [{'type': 'quotes'}], 1)
         assert 'EURUSD' in api.subscribed_symbols
-        client.subscribe_to_market_data.assert_called_with('accountId', 1, 'EURUSD')
+        client.subscribe_to_market_data.assert_called_with('accountId', 1, 'EURUSD', [{'type': 'quotes'}])
 
     @pytest.mark.asyncio
     async def test_unsubscribe_from_market_data(self):
         """Should unsubscribe from market data."""
         client.subscribe_to_market_data = AsyncMock()
         client.unsubscribe_from_market_data = AsyncMock()
-        await api.subscribe_to_market_data('EURUSD', 1)
+        await api.subscribe_to_market_data('EURUSD', [{'type': 'quotes'}], 1)
         assert 'EURUSD' in api.subscribed_symbols
-        await api.unsubscribe_from_market_data('EURUSD', 1)
+        await api.unsubscribe_from_market_data('EURUSD', [{'type': 'quotes'}], 1)
         assert 'EURUSD' not in api.subscribed_symbols
-        client.unsubscribe_from_market_data.assert_called_with('accountId', 1, 'EURUSD')
+        client.unsubscribe_from_market_data.assert_called_with('accountId', 1, 'EURUSD', [{'type': 'quotes'}])
+
+    @pytest.mark.asyncio
+    async def test_unsubscribe_during_subscription_downgrade(self):
+        """Should unsubscribe during market data subscription downgrade."""
+        api.subscribe_to_market_data = AsyncMock()
+        api.unsubscribe_from_market_data = AsyncMock()
+        await api.on_subscription_downgraded('1:ps-mpa-1', 'EURUSD', None, [{'type': 'ticks'}, {'type': 'books'}])
+        api.unsubscribe_from_market_data.assert_called_with('EURUSD', [{'type': 'ticks'}, {'type': 'books'}])
+        api.subscribe_to_market_data.assert_not_called()
+
+    @pytest.mark.asyncio
+    async def test_update_market_data_subscription_on_downgrade(self):
+        """Should update market data subscription on downgrade."""
+        api.subscribe_to_market_data = AsyncMock()
+        api.unsubscribe_from_market_data = AsyncMock()
+        await api.on_subscription_downgraded('1:ps-mpa-1', 'EURUSD',
+                                             [{'type': 'quotes', 'intervalInMilliseconds': 30000}])
+        api.subscribe_to_market_data.assert_called_with('EURUSD', [{'type': 'quotes', 'intervalInMilliseconds': 30000}])
+        api.unsubscribe_from_market_data.assert_not_called()
+
+    @pytest.mark.asyncio
+    async def test_retrieve_symbols(self):
+        """Should retrieve symbols."""
+        symbols = ['EURUSD']
+        client.get_symbols = AsyncMock(return_value=symbols)
+        actual = await api.get_symbols()
+        assert actual == symbols
+        client.get_symbols.assert_called_with('accountId')
 
     @pytest.mark.asyncio
     async def test_retrieve_symbol_specification(self):
@@ -811,6 +818,73 @@ class TestMetaApiConnection:
 
         client.get_symbol_price.assert_called_with('accountId', 'AUDNZD')
 
+    @pytest.mark.asyncio
+    async def test_retrieve_current_candle(self):
+        """Should retrieve current candle."""
+        candle = {
+            'symbol': 'AUDNZD',
+            'timeframe': '15m',
+            'time': '2020-04-07T03:45:00.000Z',
+            'brokerTime': '2020-04-07 06:45:00.000',
+            'open': 1.03297,
+            'high': 1.06309,
+            'low': 1.02705,
+            'close': 1.043,
+            'tickVolume': 1435,
+            'spread': 17,
+            'volume': 345
+        }
+        client.get_candle = AsyncMock(return_value=candle)
+        actual = await api.get_candle('AUDNZD', '15m')
+        candle['time'] = date(candle['time'])
+        assert actual == candle
+        client.get_candle.assert_called_with('accountId', 'AUDNZD', '15m')
+
+    @pytest.mark.asyncio
+    async def test_retrieve_latest_tick(self):
+        """Should retrieve latest tick."""
+        tick = {
+            'symbol': 'AUDNZD',
+            'time': '2020-04-07T03:45:00.000Z',
+            'brokerTime': '2020-04-07 06:45:00.000',
+            'bid': 1.05297,
+            'ask': 1.05309,
+            'last': 0.5298,
+            'volume': 0.13,
+            'side': 'buy'
+        }
+        client.get_tick = AsyncMock(return_value=tick)
+        actual = await api.get_tick('AUDNZD')
+        tick['time'] = date(tick['time'])
+        assert actual == tick
+        client.get_tick.assert_called_with('accountId', 'AUDNZD')
+
+    @pytest.mark.asyncio
+    async def test_retrieve_latest_order_book(self):
+        """Should retrieve latest order book."""
+        book = {
+            'symbol': 'AUDNZD',
+            'time': '2020-04-07T03:45:00.000Z',
+            'brokerTime': '2020-04-07 06:45:00.000',
+            'book': [
+                {
+                    'type': 'BOOK_TYPE_SELL',
+                    'price': 1.05309,
+                    'volume': 5.67
+                },
+                {
+                    'type': 'BOOK_TYPE_BUY',
+                    'price': 1.05297,
+                    'volume': 3.45
+                }
+            ]
+        }
+        client.get_book = AsyncMock(return_value=book)
+        actual = await api.get_book('AUDNZD')
+        book['time'] = date(book['time'])
+        assert actual == book
+        client.get_book.assert_called_with('accountId', 'AUDNZD')
+
     @pytest.mark.asyncio
     async def test_save_uptime_stats(self):
         """Should save uptime stats to the server."""
@@ -853,11 +927,13 @@ class TestMetaApiConnection:
         with patch('lib.metaApi.metaApiConnection.random_id', return_value='synchronizationId'):
             client.synchronize = AsyncMock()
             api = MetaApiConnection(client, account, None, MagicMock())
-            await api.history_storage.on_history_order_added(1, {'doneTime': date('2020-01-01T00:00:00.000Z')})
-            await api.history_storage.on_deal_added(1, {'time': date('2020-01-02T00:00:00.000Z')})
-            await api.on_connected(1, 1)
-            client.synchronize.assert_called_with('accountId', 1, 'synchronizationId', date('2020-01-01T00:00:00.000Z'),
-                                                  date('2020-01-02T00:00:00.000Z'))
+            await api.history_storage.on_history_order_added('1:ps-mpa-1',
+                                                             {'doneTime': date('2020-01-01T00:00:00.000Z')})
+            await api.history_storage.on_deal_added('1:ps-mpa-1', {'time': date('2020-01-02T00:00:00.000Z')})
+            await api.on_connected('1:ps-mpa-1', 1)
+            await asyncio.sleep(0.05)
+            client.synchronize.assert_called_with('accountId', 1, 'ps-mpa-1', 'synchronizationId',
+                                                  date('2020-01-01T00:00:00.000Z'), date('2020-01-02T00:00:00.000Z'))
 
     @pytest.mark.asyncio
     async def test_maintain_sync(self):
@@ -865,27 +941,46 @@ class TestMetaApiConnection:
         with patch('lib.metaApi.metaApiConnection.random_id', return_value='synchronizationId'):
             client.synchronize = AsyncMock(side_effect=[Exception('test error'), None])
             api = MetaApiConnection(client, account, None, MagicMock())
-            await api.history_storage.on_history_order_added(1, {'doneTime': date('2020-01-01T00:00:00.000Z')})
-            await api.history_storage.on_deal_added(1, {'time': date('2020-01-02T00:00:00.000Z')})
-            await api.on_connected(1, 1)
-            client.synchronize.assert_called_with('accountId', 1, 'synchronizationId', date('2020-01-01T00:00:00.000Z'),
-                                                  date('2020-01-02T00:00:00.000Z'))
+            await api.history_storage.on_history_order_added('1:ps-mpa-1',
+                                                             {'doneTime': date('2020-01-01T00:00:00.000Z')})
+            await api.history_storage.on_deal_added('1:ps-mpa-1', {'time': date('2020-01-02T00:00:00.000Z')})
+            await api.on_connected('1:ps-mpa-1', 1)
+            await asyncio.sleep(0.05)
+            client.synchronize.assert_called_with('accountId', 1,  'ps-mpa-1', 'synchronizationId',
+                                                  date('2020-01-01T00:00:00.000Z'), date('2020-01-02T00:00:00.000Z'))
+
+    @pytest.mark.asyncio
+    async def test_not_sync_if_connection_closed(self):
+        """Should not synchronize if connection is closed."""
+        with patch('lib.metaApi.metaApiConnection.random_id', return_value='synchronizationId'):
+            client.synchronize = AsyncMock()
+            client.unsubscribe = AsyncMock()
+            api = MetaApiConnection(client, account, None, MagicMock())
+            await api.history_storage.on_history_order_added('1:ps-mpa-1',
+                                                             {'doneTime': date('2020-01-01T00:00:00.000Z')})
+            await api.history_storage.on_deal_added('1:ps-mpa-1', {'time': date('2020-01-02T00:00:00.000Z')})
+            await api.close()
+            await api.on_connected('1:ps-mpa-1', 1)
+            client.synchronize.assert_not_called()
 
     @pytest.mark.asyncio
     async def test_restore_market_data_subs_on_sync(self):
         """Should restore market data subscriptions on synchronization."""
-        with patch('lib.metaApi.metaApiConnection.random_id', return_value='synchronizationId'):
-            client.synchronize = AsyncMock()
-            client.subscribe_to_market_data = AsyncMock()
-            api = MetaApiConnection(client, account, None, MagicMock())
-            await api.history_storage.on_history_order_added(1, {'doneTime': date('2020-01-01T00:00:00.000Z')})
-            await api.history_storage.on_deal_added(1, {'time': date('2020-01-02T00:00:00.000Z')})
-            await api.subscribe_to_market_data('EURUSD', 1)
-            await api.on_connected(1, 1)
-            client.synchronize.assert_called_with('accountId', 1, 'synchronizationId', date('2020-01-01T00:00:00.000Z'),
-                                                  date('2020-01-02T00:00:00.000Z'))
-            client.subscribe_to_market_data.assert_called_with('accountId', 1, 'EURUSD')
-            assert client.subscribe_to_market_data.call_count == 2
+        def get_price(symbol):
+            if symbol == 'EURUSD':
+                return {'symbol': symbol}
+            else:
+                return None
+
+        client.subscribe_to_market_data = AsyncMock()
+        api.terminal_state.price = get_price
+        await api.subscribe_to_market_data('EURUSD')
+        await api.subscribe_to_market_data('AUDNZD')
+        client.subscribe_to_market_data = AsyncMock()
+        await api.on_account_information_updated('1:ps-mpa-1', {})
+        await asyncio.sleep(0.05)
+        assert client.subscribe_to_market_data.call_count == 1
+        client.subscribe_to_market_data.assert_called_with('accountId', 1, 'AUDNZD', None)
 
     @pytest.mark.asyncio
     async def test_unsubscribe_from_events_on_close(self):
@@ -904,7 +999,7 @@ class TestMetaApiConnection:
     @pytest.mark.asyncio
     async def test_wait_sync_complete_user_mode(self):
         """Should wait until synchronization complete."""
-        assert not (await api.is_synchronized(1))
+        assert not (await api.is_synchronized('1:ps-mpa-1'))
         api._historyStorage.update_disk_storage = AsyncMock()
         try:
             await api.wait_synchronized({'applicationPattern': 'app.*', 'synchronizationId': 'synchronizationId',
@@ -912,14 +1007,14 @@ class TestMetaApiConnection:
             raise Exception('TimeoutError is expected')
         except Exception as err:
             assert err.__class__.__name__ == 'TimeoutException'
-        await api.on_order_synchronization_finished(1, 'synchronizationId')
-        await api.on_deal_synchronization_finished(1, 'synchronizationId')
+        await api.on_order_synchronization_finished('1:ps-mpa-1', 'synchronizationId')
+        await api.on_deal_synchronization_finished('1:ps-mpa-1', 'synchronizationId')
         promise = api.wait_synchronized({'applicationPattern': 'app.*', 'synchronizationId': 'synchronizationId',
                                          'timeoutInSeconds': 1, 'intervalInMilliseconds': 10})
         start_time = datetime.now()
         await promise
         assert pytest.approx(10, 10) == (datetime.now() - start_time).seconds * 1000
-        assert (await api.is_synchronized(1, 'synchronizationId'))
+        assert (await api.is_synchronized('1:ps-mpa-1', 'synchronizationId'))
 
     @pytest.mark.asyncio
     async def test_time_out_waiting_for_sync(self):
@@ -930,14 +1025,7 @@ class TestMetaApiConnection:
             raise Exception('TimeoutError is expected')
         except Exception as err:
             assert err.__class__.__name__ == 'TimeoutException'
-        assert not (await api.is_synchronized(1, 'synchronizationId'))
-
-    @pytest.mark.asyncio
-    async def test_subscribe_to_terminal_on_reconnect(self):
-        """Should subscribe to terminal on reconnect."""
-        api.subscribe = AsyncMock()
-        await api.on_reconnected()
-        api.subscribe.assert_called_with()
+        assert not (await api.is_synchronized('1:ps-mpa-1', 'synchronizationId'))
 
     @pytest.mark.asyncio
     async def test_load_history_storage_from_disk(self):
@@ -945,3 +1033,25 @@ class TestMetaApiConnection:
         api._historyStorage.initialize = AsyncMock()
         await api.initialize()
         api._historyStorage.initialize.assert_called()
+
+    @pytest.mark.asyncio
+    async def test_disconnect(self):
+        """Should set synchronized false on disconnect."""
+        client.synchronize = AsyncMock()
+        await api.on_connected('1:ps-mpa-1', 2)
+        await asyncio.sleep(0.05)
+        assert api.synchronized
+        client.subscribe = AsyncMock()
+        account.reload = AsyncMock()
+        await api.on_disconnected('1:ps-mpa-1')
+        assert not api.synchronized
+
+    @pytest.mark.asyncio
+    async def test_on_stream_closed(self):
+        """Should delete state if stream closed."""
+        client.synchronize = AsyncMock()
+        await api.on_connected('1:ps-mpa-1', 2)
+        await asyncio.sleep(0.05)
+        assert api.synchronized
+        await api.on_stream_closed('1:ps-mpa-1')
+        assert not api.synchronized
