@@ -112,7 +112,7 @@ class TerminalState(SynchronizationListener):
         return state['pricesBySymbol'][symbol] if \
             (symbol in state['pricesBySymbol']) else None
 
-    async def on_connected(self, instance_index: int, replicas: int):
+    async def on_connected(self, instance_index: str, replicas: int):
         """Invoked when connection to MetaTrader terminal established.
 
         Args:
@@ -123,9 +123,8 @@ class TerminalState(SynchronizationListener):
             A coroutine which resolves when the asynchronous event is processed.
         """
         self._get_state(instance_index)['connected'] = True
-        self._reset_disconnect_timer(instance_index)
 
-    async def on_disconnected(self, instance_index: int):
+    async def on_disconnected(self, instance_index: str):
         """Invoked when connection to MetaTrader terminal terminated.
 
         Args:
@@ -138,16 +137,7 @@ class TerminalState(SynchronizationListener):
         state['connected'] = False
         state['connectedToBroker'] = False
 
-    def _reset_disconnect_timer(self, instance_index: int):
-        async def disconnect():
-            await asyncio.sleep(60)
-            await self.on_disconnected(instance_index)
-
-        if hasattr(self, '_status_timer'):
-            self._status_timer.cancel()
-        self._status_timer = asyncio.create_task(disconnect())
-
-    async def on_broker_connection_status_changed(self, instance_index: int, connected: bool):
+    async def on_broker_connection_status_changed(self, instance_index: str, connected: bool):
         """Invoked when broker connection status have changed.
 
         Args:
@@ -158,9 +148,8 @@ class TerminalState(SynchronizationListener):
             A coroutine which resolves when the asynchronous event is processed.
         """
         self._get_state(instance_index)['connectedToBroker'] = connected
-        self._reset_disconnect_timer(instance_index)
 
-    async def on_synchronization_started(self, instance_index: int):
+    async def on_synchronization_started(self, instance_index: str):
         """Invoked when MetaTrader terminal state synchronization is started.
 
         Args:
@@ -181,7 +170,7 @@ class TerminalState(SynchronizationListener):
         state['ordersInitialized'] = False
         state['positionsInitialized'] = False
 
-    async def on_account_information_updated(self, instance_index: int,
+    async def on_account_information_updated(self, instance_index: str,
                                              account_information: MetatraderAccountInformation):
         """Invoked when MetaTrader position is updated.
 
@@ -194,7 +183,7 @@ class TerminalState(SynchronizationListener):
         """
         self._get_state(instance_index)['accountInformation'] = account_information
 
-    async def on_positions_replaced(self, instance_index: int, positions: List[MetatraderPosition]):
+    async def on_positions_replaced(self, instance_index: str, positions: List[MetatraderPosition]):
         """Invoked when the positions are replaced as a result of initial terminal state synchronization.
 
         Args:
@@ -209,7 +198,7 @@ class TerminalState(SynchronizationListener):
         state['removedPositions'] = {}
         state['positionsInitialized'] = True
 
-    async def on_position_updated(self, instance_index: int, position: MetatraderPosition):
+    async def on_position_updated(self, instance_index: str, position: MetatraderPosition):
         """Invoked when MetaTrader position is updated.
 
         Args:
@@ -229,7 +218,7 @@ class TerminalState(SynchronizationListener):
         if (not is_exists) and (position['id'] not in state['removedPositions']):
             state['positions'].append(position)
 
-    async def on_position_removed(self, instance_index: int, position_id: str):
+    async def on_position_removed(self, instance_index: str, position_id: str):
         """Invoked when MetaTrader position is removed.
 
         Args:
@@ -250,7 +239,7 @@ class TerminalState(SynchronizationListener):
         else:
             state['positions'] = list(filter(lambda p: p['id'] != position_id, state['positions']))
 
-    async def on_orders_replaced(self, instance_index: int, orders: List[MetatraderOrder]):
+    async def on_orders_replaced(self, instance_index: str, orders: List[MetatraderOrder]):
         """Invoked when the orders are replaced as a result of initial terminal state synchronization.
 
         Args:
@@ -265,7 +254,7 @@ class TerminalState(SynchronizationListener):
         state['completedOrders'] = {}
         state['ordersInitialized'] = True
 
-    async def on_order_updated(self, instance_index: int, order: MetatraderOrder):
+    async def on_order_updated(self, instance_index: str, order: MetatraderOrder):
         """Invoked when MetaTrader order is updated.
 
         Args:
@@ -285,7 +274,7 @@ class TerminalState(SynchronizationListener):
         if (not is_exists) and (order['id'] not in state['completedOrders']):
             state['orders'].append(order)
 
-    async def on_order_completed(self, instance_index: int, order_id: str):
+    async def on_order_completed(self, instance_index: str, order_id: str):
         """Invoked when MetaTrader order is completed (executed or canceled).
 
         Args:
@@ -298,7 +287,7 @@ class TerminalState(SynchronizationListener):
         state = self._get_state(instance_index)
         order = next((p for p in state['orders'] if p['id'] == order_id), None)
         if order is None:
-            for key in state['completedOrders']:
+            for key in list(state['completedOrders'].keys()):
                 e = state['completedOrders'][key]
                 if e + 5 * 60 < datetime.now().timestamp():
                     del state['completedOrders'][key]
@@ -306,28 +295,36 @@ class TerminalState(SynchronizationListener):
         else:
             state['orders'] = list(filter(lambda o: o['id'] != order_id, state['orders']))
 
-    async def on_symbol_specification_updated(self, instance_index: int, specification: MetatraderSymbolSpecification):
-        """Invoked when a symbol specification was updated
+    async def on_symbol_specifications_updated(self, instance_index: str,
+                                               specifications: List[MetatraderSymbolSpecification],
+                                               removed_symbols: List[str]):
+        """Invoked when a symbol specifications were updated.
 
         Args:
             instance_index: Index of an account instance connected.
-            specification: Updated MetaTrader symbol specification.
+            specifications: Updated MetaTrader symbol specification.
+            removed_symbols: Removed symbols.
 
         Returns:
             A coroutine which resolves when the asynchronous event is processed.
         """
         state = self._get_state(instance_index)
-        for i in range(len(state['specifications'])):
-            if state['specifications'][i]['symbol'] == specification['symbol']:
-                state['specifications'][i] = specification
-                break
-        else:
-            state['specifications'].append(specification)
-        state['specificationsBySymbol'][specification['symbol']] = specification
+        for specification in specifications:
+            for i in range(len(state['specifications'])):
+                if state['specifications'][i]['symbol'] == specification['symbol']:
+                    state['specifications'][i] = specification
+                    break
+            else:
+                state['specifications'].append(specification)
+            state['specificationsBySymbol'][specification['symbol']] = specification
+        state['specifications'] = list(filter(lambda s: s['symbol'] not in removed_symbols, state['specifications']))
+        for symbol in removed_symbols:
+            if symbol in state['specificationsBySymbol']:
+                del state['specificationsBySymbol'][symbol]
 
-    async def on_symbol_prices_updated(self, instance_index: int, prices: List[MetatraderSymbolPrice],
+    async def on_symbol_prices_updated(self, instance_index: str, prices: List[MetatraderSymbolPrice],
                                        equity: float = None, margin: float = None, free_margin: float = None,
-                                       margin_level: float = None):
+                                       margin_level: float = None, account_currency_exchange_rate: float = None):
         """Invoked when prices for several symbols were updated.
 
         Args:
@@ -337,6 +334,7 @@ class TerminalState(SynchronizationListener):
             margin: Margin used.
             free_margin: Free margin.
             margin_level: Margin level calculated as % of equity/margin.
+            account_currency_exchange_rate: Current exchange rate of account currency into USD.
 
         Returns:
             A coroutine which resolves when the asynchronous event is processed.
@@ -368,10 +366,26 @@ class TerminalState(SynchronizationListener):
                         price['bid']
         if state['accountInformation']:
             if state['positionsInitialized'] and prices_initialized:
-                state['accountInformation']['equity'] = \
-                    state['accountInformation']['balance'] + functools.reduce(
-                        lambda a, b: a + (b['unrealizedProfit'] if 'unrealizedProfit' in b else 0),
-                        state['positions'], 0)
+                if state['accountInformation']['platform'] == 'mt5':
+                    state['accountInformation']['equity'] = equity if equity is not None else \
+                        state['accountInformation']['balance'] + \
+                        functools.reduce(lambda a, b: a + round((b['unrealizedProfit'] if 'unrealizedProfit' in b and
+                                                                 b['unrealizedProfit'] is not None
+                                                                 else 0) * 100) / 100 +
+                                         round((b['swap'] if 'swap' in b and b['swap'] is not None
+                                                else 0) * 100) / 100, state['positions'], 0)
+                else:
+                    state['accountInformation']['equity'] = equity if equity is not None else \
+                        state['accountInformation']['balance'] + \
+                        functools.reduce(
+                        lambda a, b: a + round((b['swap'] if 'swap' in b and b['swap'] is not None
+                                                else 0) * 100) / 100 +
+                        round((b['commission'] if 'commission' in b and b['commission'] is not None
+                               else 0) * 100) / 100 +
+                        round((b['unrealizedProfit'] if 'unrealizedProfit' in b and b['unrealizedProfit'] is not None
+                               else 0) * 100) / 100,
+                            state['positions'], 0)
+                state['accountInformation']['equity'] = round(state['accountInformation']['equity'] * 100) / 100
             else:
                 state['accountInformation']['equity'] = equity if equity else (
                     state['accountInformation']['equity'] if 'equity' in state['accountInformation'] else None)
@@ -382,14 +396,30 @@ class TerminalState(SynchronizationListener):
             state['accountInformation']['marginLevel'] = margin_level if free_margin else (
                 state['accountInformation']['marginLevel'] if 'marginLevel' in state['accountInformation'] else None)
 
+    async def on_stream_closed(self, instance_index: str):
+        """Invoked when a stream for an instance index is closed.
+
+        Args:
+            instance_index: Index of an account instance connected.
+
+        Returns:
+            A coroutine which resolves when the asynchronous event is processed.
+        """
+        if instance_index in self._stateByInstanceIndex:
+            del self._stateByInstanceIndex[instance_index]
+
     def _update_position_profits(self, position: Dict, price: Dict):
         specification = self.specification(position['symbol'])
         if specification:
+            multiplier = pow(10, specification['digits'])
+            if 'profit' in position:
+                position['profit'] = round(position['profit'] * multiplier) / multiplier
             if 'unrealizedProfit' not in position or 'realizedProfit' not in position:
                 position['unrealizedProfit'] = (1 if (position['type'] == 'POSITION_TYPE_BUY') else -1) * \
                                                (position['currentPrice'] - position['openPrice']) * \
                                                position['currentTickValue'] * position['volume'] / \
                                                specification['tickSize']
+                position['unrealizedProfit'] = round(position['unrealizedProfit'] * multiplier) / multiplier
                 position['realizedProfit'] = position['profit'] - position['unrealizedProfit']
             new_position_price = price['bid'] if (position['type'] == 'POSITION_TYPE_BUY') else price['ask']
             is_profitable = (1 if (position['type'] == 'POSITION_TYPE_BUY') else -1) * (new_position_price -
@@ -398,12 +428,14 @@ class TerminalState(SynchronizationListener):
             unrealized_profit = (1 if (position['type'] == 'POSITION_TYPE_BUY') else -1) * \
                                 (new_position_price - position['openPrice']) * current_tick_value * \
                 position['volume'] / specification['tickSize']
+            unrealized_profit = round(unrealized_profit * multiplier) / multiplier
             position['unrealizedProfit'] = unrealized_profit
             position['profit'] = position['unrealizedProfit'] + position['realizedProfit']
+            position['profit'] = round(position['profit'] * multiplier) / multiplier
             position['currentPrice'] = new_position_price
             position['currentTickValue'] = current_tick_value
 
-    def _get_state(self, instance_index: int) -> TerminalStateDict:
+    def _get_state(self, instance_index: str) -> TerminalStateDict:
         if str(instance_index) not in self._stateByInstanceIndex:
             self._stateByInstanceIndex[str(instance_index)] = self._construct_terminal_state()
         return self._stateByInstanceIndex[str(instance_index)]
